/*
 * Project name:
      HTU21D click
 * Copyright:
      (c) mikroElektronika, 2014.
 * Revision History:
      20141017:
      - Initial release (DO);
 * Description:
      This project demonstrates how to read temperature and humidity from HTU21D click.
      Tempearature and humidiry results are displayed on UARTA of EasyMx PRO v7 for STM32.
 * Test configuration:
     MCU:             STM32F107VC
                      http://www.st.com/st-web-ui/static/active/en/resource/technical/document/datasheet/CD00220364.pdf
     Dev. Board:      EasyMx v7 for STM32(R) ARM(R)
                      http://www.mikroe.com/easymx-pro/stm32/
     Oscillator:      HSE-PLL, 72.000MHz
     Ext. Modules:    HTU21D click - ac:HTU21D_click
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/

 * NOTES:
      - Place HTU21D click board in the mikroBUS socket 1.
      - Turn ON UARTA SW12.1, SW12.2 switches.
*/


#define HTU21D_ADDR                 (0x40)

#define HTU21D_CLEAR_BIT            (0x40)    // Clears any pending interrupt (write 1 to clear)
#define HTU21D_WORD_BIT             (0x20)    // 1 = read/write word (rather than byte)
#define HTU21D_BLOCK_BIT            (0x10)    // 1 = using block read/write

#define TRIGGER_TEMP_MEASURE_HOLD    0xE3
#define TRIGGER_HUMD_MEASURE_HOLD    0xE5
#define TRIGGER_TEMP_MEASURE_NOHOLD  0xF3
#define TRIGGER_HUMD_MEASURE_NOHOLD  0xF5
#define WRITE_USER_REG               0xE6
#define READ_USER_REG                0xE7
#define SOFT_RESET                   0xFE

void InitMCU(){                //  Init MCU function
  UART1_Init(19200);           // Initialize UART module at 19200 bps
  I2C1_Init_Advanced(100000, &_GPIO_MODULE_I2C1_PB67);   // Configure I2C module
}

char HTU21D_Soft_Reset(){
char reg_command[1];
  reg_command[0] = SOFT_RESET;
  I2C1_Start();                                                // issue I2C start signal
  I2C1_Write(HTU21D_ADDR,reg_command, 1, END_MODE_STOP);
  Delay_ms(15);
}

void HTU21D_REG_read(){
  char reg_data[1], txt[12];
  int reg_value = 0;
  reg_data[0] = READ_USER_REG;
  I2C1_Start();
  I2C1_Write(HTU21D_ADDR, reg_data, 1, END_MODE_RESTART);
  I2C1_Read (HTU21D_ADDR, reg_data, 1, END_MODE_STOP);
  reg_value=reg_data[0];

  IntToStr(reg_value, txt);
  UART1_Write_Text(txt);
}

void HTU21D_REG_write(){
  char reg_data[1], txt[12];
  int reg_value=0;
  reg_data[0]=WRITE_USER_REG;
  reg_data[1]=0xC1;
  I2C1_Start();
  I2C1_Write(HTU21D_ADDR, reg_data, 2, END_MODE_STOP);
  reg_value = reg_data[0];
}

unsigned HTU21D_ReadValue(char regSelect){
unsigned value = 0;
char reg_data[3];

  reg_data[0] = regSelect;
  I2C1_Start();
  I2C1_Write(HTU21D_ADDR, reg_data, 1, END_MODE_RESTART);
  I2C1_Read (HTU21D_ADDR, reg_data, 3, END_MODE_STOP);
  value = ((unsigned)reg_data[0] << 8) | reg_data[1] ;
  return value & 0xFFFC;            // Clear status bits
}

void procTemperatureValue(unsigned ValueTemp){
  char txt[12];
  float calc;
  calc = -46.85 + 175.72 * ValueTemp / 65536.0;
  sprintf(txt, "%4.3f", calc);
  UART_Write_Text(txt);
}

void procHumidityValue(unsigned ValueTemp){
  char txt[12];
  float calc;
  calc = -6.0 + 125.0 * ValueTemp / 65536.0;
  sprintf(txt, "%4.3f", calc);
  UART_Write_Text(txt);
}

void main(){
  InitMCU();
  HTU21D_Soft_Reset();
  Delay_ms(50);
  UART_Write_Text("HTU21D Example\r\n\r\n");
  while(1){
    UART_Write_Text("Temperature=");
    procTemperatureValue(HTU21D_ReadValue(TRIGGER_TEMP_MEASURE_HOLD));
    UART_Write_Text("C\t");
    
    UART_Write_Text("Humidity=");
    procHumidityValue(HTU21D_ReadValue(TRIGGER_HUMD_MEASURE_HOLD));
    UART_Write_Text("%RH\r\n");
    Delay_ms(1000);
  }
}